@@ -17,16 +17,18 @@
 import typing
 
 from dataclasses import dataclass
-from datetime import datetime
+from datetime import datetime, timedelta
 from dateutil import parser
 from decimal import Decimal
 from solana.publickey import PublicKey
 
 from .account import Account
 from .accountinfo import AccountInfo
+from .addressableaccount import AddressableAccount
 from .combinableinstructions import CombinableInstructions
 from .constants import SYSTEM_PROGRAM_ADDRESS
 from .context import Context
+from .datetimes import utc_now
 from .group import Group
 from .instructions import (
     build_mango_redeem_accrued_instructions,
@@ -35,13 +37,14 @@ from .instructions import (
     build_perp_consume_events_instructions,
     build_perp_place_order_instructions,
 )
+from .layouts import layouts
 from .loadedmarket import LoadedMarket
 from .lotsizeconverter import LotSizeConverter, RaisingLotSizeConverter
 from .markets import InventorySource, MarketType, Market
 from .marketoperations import MarketInstructionBuilder, MarketOperations
+from .metadata import Metadata
 from .observables import Disposable
-from .orderbookside import PerpOrderBookSide
-from .orders import Order, OrderBook
+from .orders import Order, OrderBook, OrderType, Side
 from .perpeventqueue import (
     PerpEvent,
     PerpEventQueue,
@@ -55,7 +58,10 @@ from .tokenbank import TokenBank
 from .wallet import Wallet
 from .websocketsubscription import (
     IndividualWebSocketSubscriptionManager,
+    WebSocketAccountSubscription,
+    WebSocketSubscriptionManager,
 )
+from .version import Version
 
 
 # # 🥭 FundingRate class
@@ -114,13 +120,207 @@ class FundingRate:
             to=to_timestamp,
         )
 
+    @property
+    def extrapolated_apr(self) -> Decimal:
+        # From Max (in Discord DM):
+        # hpr * 24 * 365 = apr
+        # ...
+        # hpr * 8760 = apr
+        return self.rate * Decimal(8760)
+
+    @property
+    def extrapolated_apy(self) -> Decimal:
+        # From Max (in Discord DM):
+        # (1 + hpr) ^ 8760 = apy + 1
+        return ((Decimal(1) + self.rate) ** Decimal(8760)) - Decimal(1)
+
     def __str__(self) -> str:
-        return f"« FundingRate {self.symbol} {self.rate:,.8%}, open interest: {self.open_interest:,.8f} from: {self.from_} to {self.to} »"
+        return f"""« FundingRate {self.symbol} {self.rate:,.8%}
+    Period: {self.from_} to {self.to}
+    Open Interest: {self.open_interest:,.8f}
+    Extrapolated APR: {self.extrapolated_apr:,.8%}
+    Extrapolated APY: {self.extrapolated_apy:,.8%}
+»"""
 
     def __repr__(self) -> str:
         return f"{self}"
 
 
+# # 🥭 PerpOrderBookSide class
+#
+# `PerpOrderBookSide` holds orders for one side of a market.
+#
+class PerpOrderBookSide(AddressableAccount):
+    def __init__(
+        self,
+        account_info: AccountInfo,
+        version: Version,
+        meta_data: Metadata,
+        perp_market_details: PerpMarketDetails,
+        bump_index: Decimal,
+        free_list_len: Decimal,
+        free_list_head: Decimal,
+        root_node: Decimal,
+        leaf_count: Decimal,
+        nodes: typing.Any,
+    ) -> None:
+        super().__init__(account_info)
+        self.version: Version = version
+
+        self.meta_data: Metadata = meta_data
+        self.perp_market_details: PerpMarketDetails = perp_market_details
+        self.bump_index: Decimal = bump_index
+        self.free_list_len: Decimal = free_list_len
+        self.free_list_head: Decimal = free_list_head
+        self.root_node: Decimal = root_node
+        self.leaf_count: Decimal = leaf_count
+        self.nodes: typing.Any = nodes
+
+    @staticmethod
+    def from_layout(
+        layout: typing.Any,
+        account_info: AccountInfo,
+        version: Version,
+        perp_market_details: PerpMarketDetails,
+    ) -> "PerpOrderBookSide":
+        meta_data = Metadata.from_layout(layout.meta_data)
+        bump_index: Decimal = layout.bump_index
+        free_list_len: Decimal = layout.free_list_len
+        free_list_head: Decimal = layout.free_list_head
+        root_node: Decimal = layout.root_node
+        leaf_count: Decimal = layout.leaf_count
+        nodes: typing.Any = layout.nodes
+
+        return PerpOrderBookSide(
+            account_info,
+            version,
+            meta_data,
+            perp_market_details,
+            bump_index,
+            free_list_len,
+            free_list_head,
+            root_node,
+            leaf_count,
+            nodes,
+        )
+
+    @staticmethod
+    def parse(
+        account_info: AccountInfo, perp_market_details: PerpMarketDetails
+    ) -> "PerpOrderBookSide":
+        data = account_info.data
+        if len(data) != layouts.ORDERBOOK_SIDE.sizeof():
+            raise Exception(
+                f"PerpOrderBookSide data length ({len(data)}) does not match expected size ({layouts.ORDERBOOK_SIDE.sizeof()})"
+            )
+
+        layout = layouts.ORDERBOOK_SIDE.parse(data)
+        return PerpOrderBookSide.from_layout(
+            layout, account_info, Version.V1, perp_market_details
+        )
+
+    @staticmethod
+    def load(
+        context: Context, address: PublicKey, perp_market_details: PerpMarketDetails
+    ) -> "PerpOrderBookSide":
+        account_info = AccountInfo.load(context, address)
+        if account_info is None:
+            raise Exception(
+                f"PerpOrderBookSide account not found at address '{address}'"
+            )
+        return PerpOrderBookSide.parse(account_info, perp_market_details)
+
+    def subscribe(
+        self,
+        context: Context,
+        websocketmanager: WebSocketSubscriptionManager,
+        callback: typing.Callable[["PerpOrderBookSide"], None],
+    ) -> Disposable:
+        def __parser(account_info: AccountInfo) -> PerpOrderBookSide:
+            return PerpOrderBookSide.parse(account_info, self.perp_market_details)
+
+        subscription = WebSocketAccountSubscription(context, self.address, __parser)
+        websocketmanager.add(subscription)
+        subscription.publisher.subscribe(on_next=callback)  # type: ignore[call-arg]
+
+        return subscription
+
+    def orders(self) -> typing.Sequence[Order]:
+        if self.leaf_count == 0:
+            return []
+
+        if self.meta_data.data_type == layouts.DATA_TYPE.Bids:
+            order_side = Side.BUY
+        else:
+            order_side = Side.SELL
+
+        stack = [self.root_node]
+        orders: typing.List[Order] = []
+        while len(stack) > 0:
+            index = int(stack.pop())
+            node = self.nodes[index]
+            if node.type_name == "leaf":
+                timestamp: datetime = node.timestamp
+                expiration = Order.NoExpiration
+                if node.time_in_force != 0:
+                    expiration = timestamp + timedelta(
+                        seconds=float(node.time_in_force)
+                    )
+
+                price = node.key["price"]
+                quantity = node.quantity
+
+                decimals_differential = (
+                    self.perp_market_details.base_instrument.decimals
+                    - self.perp_market_details.quote_token.token.decimals
+                )
+                native_to_ui = Decimal(10) ** decimals_differential
+                quote_lot_size = self.perp_market_details.quote_lot_size
+                base_lot_size = self.perp_market_details.base_lot_size
+                actual_price = price * (quote_lot_size / base_lot_size) * native_to_ui
+
+                base_factor = (
+                    Decimal(10) ** self.perp_market_details.base_instrument.decimals
+                )
+                actual_quantity = (
+                    quantity * self.perp_market_details.base_lot_size
+                ) / base_factor
+
+                orders += [
+                    Order(
+                        int(node.key["order_id"]),
+                        node.client_order_id,
+                        node.owner,
+                        order_side,
+                        actual_price,
+                        actual_quantity,
+                        OrderType.UNKNOWN,
+                        timestamp=timestamp,
+                        expiration=expiration,
+                    )
+                ]
+            elif node.type_name == "inner":
+                if order_side == Side.BUY:
+                    stack = [*stack, node.children[0], node.children[1]]
+                else:
+                    stack = [*stack, node.children[1], node.children[0]]
+        return orders
+
+    def __str__(self) -> str:
+        nodes = "\n        ".join(
+            [str(node).replace("\n", "\n        ") for node in self.orders()]
+        )
+        return f"""« PerpOrderBookSide {self.version} [{self.address}]
+    {self.meta_data}
+    Perp Market: {self.perp_market_details}
+    Bump Index: {self.bump_index}
+    Free List: {self.free_list_head} (head) {self.free_list_len} (length)
+    Root Node: {self.root_node}
+    Leaf Count: {self.leaf_count}
+        {nodes}
+»"""
+
+
 # # 🥭 PerpMarket class
 #
 # This class encapsulates our knowledge of a Mango perps market.
@@ -465,11 +665,11 @@ class PerpMarketOperations(MarketOperations):
     def load_orderbook(self) -> OrderBook:
         return self.perp_market.fetch_orderbook(self.context)
 
-    def load_my_orders(self, include_expired: bool = False) -> typing.Sequence[Order]:
+    def load_my_orders(
+        self, cutoff: typing.Optional[datetime] = utc_now()
+    ) -> typing.Sequence[Order]:
         orderbook: OrderBook = self.load_orderbook()
-        return orderbook.all_orders_for_owner(
-            self.account.address, include_expired=include_expired
-        )
+        return orderbook.all_orders_for_owner(self.account.address, cutoff)
 
     def _build_crank(
         self, limit: Decimal = Decimal(32), add_self: bool = False
